<?php
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}
/**
 * Define the post functionality.
 *
 *
 * @since      1.0.0
 * @package    WP_Video_Encoder
 * @subpackage WP_Video_Encoder/includes
 * @author     phpface <nttoanbrvt@gmail.com>
 */
class WP_Video_Encoder_Post {

	private $settings;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $settings ) {
		$this->settings = $settings;
	}

	/**
	 *
	 * Set post thumbnail
	 * 
	 * @param image path $image
	 * @param int $post_id
	 *
	 * @return  array()
	 *
	 * @since  1.0.0
	 * 
	 */
	private function set_attachment_image( $image, $attachment_id, $set_thumbnail = false ){

		$filetype = wp_check_filetype( basename( $image ), null );

		$thumbnail_id = wp_insert_attachment(
			array(
				'post_mime_type'	=> $filetype['type'],
				'post_title'		=> preg_replace( '/\.[^.]+$/', '', basename( $image ) ),
				'post_status'		=> 'inherit',
				'post_author'		=>	get_post( $attachment_id )->post_author
			), 
			$image
		);

		if ( ! function_exists( 'wp_generate_attachment_metadata' ) ) {
		    include( ABSPATH . 'wp-admin/includes/image.php' );
		}		
		
		wp_update_attachment_metadata( $thumbnail_id, wp_generate_attachment_metadata( $thumbnail_id, $image ) );

		if( $set_thumbnail ){
			set_post_thumbnail( $attachment_id, $thumbnail_id );
		}

		wp_update_post( array(
			'ID'			=>	(int)$thumbnail_id,
			'post_parent'	=>	(int)$attachment_id
		) );

		return compact( 'attachment_id', 'thumbnail_id' );
	}

	/**
	 * 
	 *
	 * Create image from given attachment ID
	 * 
	 * @param  int $attachment_id
	 * @return $this->create_post_thumbnail() or false
	 *
	 * @since  1.0.0
	 * 
	 */
	public function generate_attachment_image( $attachment_id ){

		if( ! $this->settings['auto_generate_image'] ){
			return $attachment_id;
		}

		// Do nothing if attachment isn't video
		if( ! wp_attachment_is( 'video', $attachment_id ) ){
			return new WP_Error(
				'attachment_not_found',
				esc_html__( 'Invalid Video Format or Attachment was not found', 'wp-video-encoder' )
			);
		}
		
		$encoder = new WP_Video_Encoder_Encoder( 
			get_attached_file( $attachment_id ), 
			$this->settings['bin_path'] 
		);

		$image = $encoder->generate_image();

		if( file_exists( $image ) ){
			return $this->set_attachment_image( $image, $attachment_id, true );
		}

		return new WP_Error(
			'undefined_error',
			esc_html__( 'Undefined Error', 'wp-video-encoder' )
		);
	}

	/**
	 *
	 * Generate webp image
	 * 
	 * @param  int $attachment_id
	 *
	 * 
	 */
	public function generate_attachment_image_webp( $attachment_id ){

		if( ! $this->settings['auto_generate_webp_image'] ){
			return $attachment_id;
		}

		// Do nothing if attachment isn't video
		if( ! wp_attachment_is( 'video', $attachment_id ) ){
			return new WP_Error(
				'attachment_not_found',
				esc_html__( 'Invalid Video Format or Attachment was not found', 'wp-video-encoder' )
			);
		}
		
		$encoder = new WP_Video_Encoder_Encoder( 
			get_attached_file( $attachment_id ), 
			$this->settings['bin_path'] 
		);

		$image = $encoder->generate_image_webp( array(
			'resolution'	=>	$this->settings['webp_resolution']
		) );

		if( file_exists( $image ) ){
			$results = $this->set_attachment_image( $image, $attachment_id );

			if( is_array( $results ) ){
				update_post_meta( (int)$attachment_id, '_thumbnail_id_2', (int)$results['thumbnail_id'] );
			}

			return $results;
		}

		return false;
	}

	/**
	 * 
	 *
	 * Filter the video attachment URL
	 * point to playlist.m3u8 URL if found
	 * 
	 * @param  string $url
	 * @param  int $attachment_id
	 * @return string
	 *
	 *
	 * @since  1.0.0
	 * 
	 */
	public function filter_get_attachment_url( $url, $attachment_id ){

		// Do nothing if attachment isn't video
		if( ! wp_attachment_is( 'video', $attachment_id ) ){
			return $url;
		}

		$file_path = get_attached_file( $attachment_id );

		$file_info = pathinfo( $file_path );

		$encoder = new WP_Video_Encoder_Encoder( $file_path, $this->settings['bin_path'] );

		if( file_exists( $encoder->get_encode_playlist_file() ) ){
			return trailingslashit( dirname( $url )) . sanitize_file_name( $file_info['filename'] ) . '/' . $encoder->get_file_master();
		}

		return $url;
	}

	/**
	 *
	 * Run before an attachment is deleted
	 * 
	 * @param  int $attachment_id [description]
	 * @param  WP_Post $attachment
	 *
	 * @since 1.0.0
	 * 
	 */
	public function delete_attachment( $attachment_id, $attachment ){

		if( wp_attachment_is( 'video',  $attachment_id ) ){

			/**
			 * Delete all encoded files such as log, ts and m3u8
			 */
			$encoder = new WP_Video_Encoder_Encoder( get_attached_file( $attachment_id ) );

			$encoder->delete_encoded_files();

			/**
			 * Remove queue item
			 */
			$queue = new WP_Video_Encoder_Queue();

			$queue->delete_queue_item( $attachment_id );
		}
	}

	/**
	 *
	 * Get encode status template
	 * 
	 * @param  int $attachment_id
	 * @return HTML
	 *
	 * @since  1.0.0
	 * 
	 */
	public function get_encode_status( $attachment_id ){
		load_template( WP_VIDEO_ENCODER_ADMIN_PATH . 'partials/encode-status.php', false, array(
			'post_id'	=>	$attachment_id
		) );
	}

	/**
	 *
	 * Filter video extensions
	 * 
	 * @param  array $formats
	 * @return array $formats
	 *
	 * @since  1.0.5
	 */
	public function filter_wp_video_extensions( $formats ){
		$_formats = $this->settings['allow_formats'];

		if( empty( $_formats ) ){
			return $formats;
		}

		return array_map( 'trim', explode(',', $_formats ) );
	}

}